# Window: Graphs of money, crime, etc, for Unix Micropolis
#
# Micropolis, Unix Version.  This game was released for the Unix platform
# in or about 1990 and has been modified for inclusion in the One Laptop
# Per Child program.  Copyright (C) 1989 - 2007 Electronic Arts Inc.  If
# you need assistance with this program, you may contact:
#   http://wiki.laptop.org/go/Micropolis  or email  micropolis@laptop.org.
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or (at
# your option) any later version.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.  You should have received a
# copy of the GNU General Public License along with this program.  If
# not, see <http://www.gnu.org/licenses/>.
# 
#             ADDITIONAL TERMS per GNU GPL Section 7
# 
# No trademark or publicity rights are granted.  This license does NOT
# give you any right, title or interest in the trademark SimCity or any
# other Electronic Arts trademark.  You may not distribute any
# modification of this program using the trademark SimCity or claim any
# affliation or association with Electronic Arts Inc. or its employees.
# 
# Any propagation or conveyance of this program must include this
# copyright notice and these terms.
# 
# If you convey this program (or any modifications of it) and assume
# contractual liability for the program to recipients of it, you agree
# to indemnify Electronic Arts for any liability that those contractual
# assumptions impose on Electronic Arts.
# 
# You may not misrepresent the origins of this program; modified
# versions of the program must be marked as such and not identified as
# the original program.
# 
# This disclaimer supplements the one included in the General Public
# License.  TO THE FULLEST EXTENT PERMISSIBLE UNDER APPLICABLE LAW, THIS
# PROGRAM IS PROVIDED TO YOU "AS IS," WITH ALL FAULTS, WITHOUT WARRANTY
# OF ANY KIND, AND YOUR USE IS AT YOUR SOLE RISK.  THE ENTIRE RISK OF
# SATISFACTORY QUALITY AND PERFORMANCE RESIDES WITH YOU.  ELECTRONIC ARTS
# DISCLAIMS ANY AND ALL EXPRESS, IMPLIED OR STATUTORY WARRANTIES,
# INCLUDING IMPLIED WARRANTIES OF MERCHANTABILITY, SATISFACTORY QUALITY,
# FITNESS FOR A PARTICULAR PURPOSE, NONINFRINGEMENT OF THIRD PARTY
# RIGHTS, AND WARRANTIES (IF ANY) ARISING FROM A COURSE OF DEALING,
# USAGE, OR TRADE PRACTICE.  ELECTRONIC ARTS DOES NOT WARRANT AGAINST
# INTERFERENCE WITH YOUR ENJOYMENT OF THE PROGRAM; THAT THE PROGRAM WILL
# MEET YOUR REQUIREMENTS; THAT OPERATION OF THE PROGRAM WILL BE
# UNINTERRUPTED OR ERROR-FREE, OR THAT THE PROGRAM WILL BE COMPATIBLE
# WITH THIRD PARTY SOFTWARE OR THAT ANY ERRORS IN THE PROGRAM WILL BE
# CORRECTED.  NO ORAL OR WRITTEN ADVICE PROVIDED BY ELECTRONIC ARTS OR
# ANY AUTHORIZED REPRESENTATIVE SHALL CREATE A WARRANTY.  SOME
# JURISDICTIONS DO NOT ALLOW THE EXCLUSION OF OR LIMITATIONS ON IMPLIED
# WARRANTIES OR THE LIMITATIONS ON THE APPLICABLE STATUTORY RIGHTS OF A
# CONSUMER, SO SOME OR ALL OF THE ABOVE EXCLUSIONS AND LIMITATIONS MAY
# NOT APPLY TO YOU.
global GraphWindows
set n [Unique]
set parent [WindowLink $head.col2]
set win $parent.graph$n
set GraphWindows [linsert $GraphWindows 0 $win]

LinkWindow $head.graph $win
LinkWindow $win.head $head
LinkWindow $win.parent $parent
LinkWindow $win.top $win

catch "destroy $win"
frame $win

SetHelp $win Graph

bind $win <Visibility> {[WindowLink %W.graphview] Visible [string compare %s FullyObscured]}
bind $win <Map> {[WindowLink %W.graphview] Visible 1}
bind $win <Unmap> {[WindowLink %W.graphview] Visible 0}

#wm title $win {Micropolis Graph}
#wm iconname $win {Micropolis Graph}
#wm group $win $head
#wm geometry $win 550x200+440+730
#wm positionfrom $win user
#wm withdraw $win
#wm maxsize $win 1000 500
#wm minsize $win 150 85
#wm protocol $win delete "DeleteWindow graph GraphWindows"


frame $win.centerframe\
  -borderwidth 1\
  -relief raised

SetHelp $win.centerframe Graph.View

graphview $win.centerframe.graph\
  -font [Font $win Small]
LinkWindow $win.centerframe.graph.pallet $win.leftframe
LinkWindow $win.graphview $win.centerframe.graph

pack append $win.centerframe\
  $win.centerframe.graph {top frame center fill expand}

frame $win.leftframe\
  -borderwidth 1\
  -relief raised

frame $win.leftframe.left\
  -borderwidth 0

button $win.leftframe.left.res\
  -bitmap "@images/grres.xpm"\
  -font [Font $win Medium]\
  -borderwidth 0\
  -relief flat\
  -padx 0\
  -pady 0\
  -text {}\
  -command "GraphPallet $win 0"

SetHelp $win.leftframe.left.res Graph.Res

button $win.leftframe.left.com\
  -bitmap "@images/grcom.xpm"\
  -font [Font $win Medium]\
  -borderwidth 0\
  -relief flat\
  -padx 0\
  -pady 0\
  -text {}\
  -command "GraphPallet $win 1"

SetHelp $win.leftframe.left.com Graph.Com

button $win.leftframe.left.ind\
  -bitmap "@images/grind.xpm"\
  -font [Font $win Medium]\
  -borderwidth 0\
  -relief flat\
  -padx 0\
  -pady 0\
  -text {}\
  -command "GraphPallet $win 2"

SetHelp $win.leftframe.left.ind Graph.Ind

pack append $win.leftframe.left\
  $win.leftframe.left.res	{top frame center padx 2 pady 2} \
  $win.leftframe.left.com	{top frame center padx 2 pady 2} \
  $win.leftframe.left.ind	{top frame center padx 2 pady 2}

frame $win.leftframe.right\
  -borderwidth 0

button $win.leftframe.right.money\
  -bitmap "@images/grmony.xpm"\
  -font [Font $win Medium]\
  -borderwidth 0\
  -relief flat\
  -padx 0\
  -pady 0\
  -text {}\
  -command "GraphPallet $win 3"

SetHelp $win.leftframe.right.money Graph.Money

button $win.leftframe.right.crime\
  -bitmap "@images/grcrim.xpm"\
  -font [Font $win Medium]\
  -borderwidth 0\
  -relief flat\
  -padx 0\
  -pady 0\
  -text {}\
  -command "GraphPallet $win 4"

SetHelp $win.leftframe.right.crime Graph.Crime

button $win.leftframe.right.pollution\
  -bitmap "@images/grpoll.xpm"\
  -font [Font $win Medium]\
  -borderwidth 0\
  -relief flat\
  -padx 0\
  -pady 0\
  -text {}\
  -command "GraphPallet $win 5"

SetHelp $win.leftframe.right.pollution Graph.Pollution

pack append $win.leftframe.right\
  $win.leftframe.right.money		{top frame center padx 2 pady 2} \
  $win.leftframe.right.crime		{top frame center padx 2 pady 2} \
  $win.leftframe.right.pollution	{top frame center padx 2 pady 2}

frame $win.leftframe.year\
  -borderwidth 0

button $win.leftframe.year.year10\
  -bitmap "@images/gr10.xpm"\
  -font [Font $win Medium]\
  -borderwidth 0\
  -relief flat\
  -padx 0\
  -pady 0\
  -text {}\
  -command "GraphYearPallet $win 0"

SetHelp $win.leftframe.year.year10 Graph.10Years

button $win.leftframe.year.year120\
  -bitmap "@images/gr120.xpm"\
  -font [Font $win Medium]\
  -borderwidth 0\
  -relief flat\
  -padx 0\
  -pady 0\
  -text {}\
  -command "GraphYearPallet $win 1"

SetHelp $win.leftframe.year.year120 Graph.120Years

pack append $win.leftframe.year\
  $win.leftframe.year.year10	{top frame center} \
  $win.leftframe.year.year120	{top frame center} \

pack append $win.leftframe\
  $win.leftframe.year		{top frame center padx 2 pady 2} \
  $win.leftframe.left		{left frame center} \
  $win.leftframe.right		{right frame center} \


frame $win.bottomframe\
  -borderwidth 0

SetHelp $win.bottomframe Graph.Dismiss

button $win.bottomframe.button\
  -font [Font $win Large]\
  -text {Dismiss Graph}\
  -borderwidth 1 \
  -command "pack unpack $win"

pack append $win.bottomframe\
  $win.bottomframe.button   {top frame center expand fillx}

pack append $win\
  $win.bottomframe	{bottom frame center fillx}\
  $win.leftframe	{left frame center filly} \
  $win.centerframe	{right frame center expand fill}

InitGraph $win

update idletasks
